from tkinter import *
import threading,time,os
from XenonUI.XUIlib.imgtool import add_image
from XenonUI.XUIlib.ImageLibConfig import Image_XUI as info_image, img_encode as img_encode

#XUI V1.1
#Junxiang H. 2023.03.07

class VerticalScrolledFrame(Frame):
	"""A pure Tkinter scrollable frame that actually works!
	* Use the 'interior' attribute to place widgets inside the scrollable frame.
	* Construct and pack/place/grid normally.
	* This frame only allows vertical scrolling.
	"""
	def __init__(self, parent,width,height,scrollwidth,image=None, *args, **kw):
		Frame.__init__(self, parent, *args, **kw)
 
		# Create a canvas object and a vertical scrollbar for scrolling it.
		vscrollbar = Scrollbar(self,width=scrollwidth, orient=VERTICAL)
		vscrollbar.pack(fill=Y, side=RIGHT, expand=FALSE)
		hocrollbar = Scrollbar(self,width=scrollwidth, orient=HORIZONTAL)
		hocrollbar.pack(fill=X, side=BOTTOM, expand=FALSE)
		if image!=None:
			canvas = Canvas(self, bd=0, highlightthickness=0, yscrollcommand=vscrollbar.set,xscrollcommand=hocrollbar.set,width=width,height=height,image=image)
		else:
			canvas = Canvas(self, bd=0, highlightthickness=0, yscrollcommand=vscrollbar.set,xscrollcommand=hocrollbar.set,width=width,height=height) #这里设定canvas的大小是frame内界面的大小，如果frame内定义的子项的大小超过了canvas的大小，则滚动条生效
		canvas.pack(side=LEFT, fill=BOTH, expand=TRUE)
		#canvas.place(x=0,y=0,anchor="nw")
		vscrollbar.config(command=canvas.yview)
		hocrollbar.config(command=canvas.xview)
 
		# Reset the view
		canvas.xview_moveto(0)
		canvas.yview_moveto(0)
 
		# Create a frame inside the canvas which will be scrolled with it.
		self.interior = interior = Frame(canvas)
		self.interior_id = canvas.create_window(0, 0, window=interior,anchor=NW)

		# Track changes to the canvas and frame width and sync them,
		# also updating the scrollbar.
		def _configure_interior(event):
			# Update the scrollbars to match the size of the inner frame.
			size = (interior.winfo_reqwidth(), interior.winfo_reqheight())
			canvas.config(scrollregion="0 0 %s %s" % size)
		interior.bind('<Configure>', _configure_interior)
		def _configure_canvas(event):
			if interior.winfo_reqwidth() != canvas.winfo_width():
				# Update the inner frame's width to fill the canvas.
				#canvas.itemconfigure(self.interior_id, width=canvas.winfo_width())
				self.interior.config(width=canvas.winfo_width())
			if interior.winfo_reqheight() != canvas.winfo_height():
				# Update the inner frame's width to fill the canvas.
				#canvas.itemconfigure(self.interior_id, height=canvas.winfo_height())
				self.interior.config(height=canvas.winfo_height())
		canvas.bind('<Configure>', _configure_canvas)
		self.canvas=canvas
class auto_frame:
	frame_list=[]
	margin={
		"left":10,
		"right":10,
		"top":10,
		"bottom":10
	}
	scrollwidth=20
	def get_margin(self,key,margin={}):
		try:
			return margin[key]
		except:
			try:
				return self.margin[key]
			except:
				return None
	def __init__(self,frame,width,height,margin={},scrollwidth=None):
		self.ori_frame=frame
		if scrollwidth!=None:
			self.scrollwidth=scrollwidth
		self.width=width-self.get_margin("left",margin)-self.get_margin("right",margin)
		self.height=height-self.get_margin("top",margin)-self.get_margin("bottom",margin)
		self.useframe=Frame(self.ori_frame,width=self.width-self.scrollwidth,height=self.height-self.scrollwidth)
		self.useframe.place(x=self.get_margin("left",margin),y=self.get_margin("top",margin),anchor="nw")
		self.frame=VerticalScrolledFrame(self.useframe,width=self.width-self.scrollwidth,height=self.height-self.scrollwidth,scrollwidth=self.scrollwidth)
		self.frame.pack()
	def add_item(self,width,height):
		self.frame_list.append(Frame(self.frame.interior,bd=0,width=width,height=height))
	def add_item_x(self,width):
		self.add_item(width=width,height=self.height-self.scrollwidth)
		self.frame_list[-1].pack(side=LEFT)
		return self.frame_list[-1]
	def add_item_y(self,height,anchor="nw",side=None):
		self.add_item(width=self.width-self.scrollwidth,height=height)
		if side==None:
			self.frame_list[-1].pack(anchor=anchor)
		else:
			self.frame_list[-1].pack(side=side)
		return self.frame_list[-1]
class XUI:
	default_args={
		#window
		"windows_width":1440,
		"windows_height":900,
		"windows_title":"XUI v1.1",
		"windows_bg":"#fcffc8",

		#mainwin
		"mainwin_x":20,
		"mainwin_y":20,
		"mainwin_width":1400,
		"mainwin_height":860,
		"mainwin_bg":"#fcffc8",#"#fcffc8",

		#first menu bar (fmb)
		"fmb_x":0,
		"fmb_y":0,
		"fmb_width":1400,
		"fmb_height":160,
		"fmb_bg":"#dae1e3",#"#dae1e3",
		"fmb_bd":20,

		#Second menu bar (smb)
		"smb_x":0,
		"smb_y":160,
		"smb_width":300,
		"smb_height":700,
		"smb_bg":"#a9ecfd",#"#a9ecfd",
		"smb_bh":35,

		#info (info)
		"info_x":1100,
		"info_y":660,
		"info_width":300,
		"info_height":200,
		"info_bg":"#51f9bc",#"#51f9bc",
		"info_image":info_image,

		#Result output (ro)
		"ro_x":300,
		"ro_y":160,
		"ro_width":800,
		"ro_height":500,
		"ro_bg":"#fcffc8",#"#fcffc8",

		#IO (io)
		"io_x":300,
		"io_y":660,
		"io_width":800,
		"io_height":200,
		"io_bg":"#000000",#"#000000",
		"io_max_length":100,
		"io_item_height":30,

		#cpu (cpu)
		"cpu_x":1100,
		"cpu_y":160,
		"cpu_width":300,
		"cpu_height":500,
		"cpu_bg":"#ffb1f5",#"#ffb1f5",

		#item_padding (ip)
		"ip_left":10,
		"ip_right":10,
		"ip_top":10,
		"ip_bottom":10
	}
	current_fmb=None
	current_fmb2=None
	current_smb=None
	def get_pars(self,key,args={}):
		try:
			return args[key]
		except:
			try:
				return self.default_args[key]
			except:
				return None
	def __init__(self):
		img_encode()
		self.window=Tk()
		self.message=[]
		self.message_color=[]
		self.io_state=True
		threading.Thread(target=self.io_add).start()
	def build_frame(self,father_frame,key_x,key_y,key_width,key_height,key_bg,args={},anchor="nw",af=True):
		try:
			f=Frame(father_frame.canvas,bg=self.get_pars(key_bg,args),width=self.get_pars(key_width,args),height=self.get_pars(key_height,args))
		except:
			f=Frame(father_frame,bg=self.get_pars(key_bg,args),width=self.get_pars(key_width,args),height=self.get_pars(key_height,args))
		f.place(x=self.get_pars(key_x,args),y=self.get_pars(key_y,args),anchor=anchor)
		if af:
			return auto_frame(f,self.get_pars(key_width,args),self.get_pars(key_height,args))
		return f
	def build(self,**args):
		#tk window
		self.window.geometry(str(self.get_pars("windows_width",args))+"x"+str(self.get_pars("windows_height",args)))
		self.window.config(bg=self.get_pars("windows_bg",args))
		self.window.title(self.get_pars("windows_title",args))

		#main
		self.mainwin=self.build_frame(self.window,"mainwin_x","mainwin_y","mainwin_width","mainwin_height","mainwin_bg",args,af=False)

		#First menu bar
		self.fmb=self.build_frame(self.mainwin,"fmb_x","fmb_y","fmb_width","fmb_height","fmb_bg",args)
		self.fmb_item=[]

		#Second menu bar
		self.smb=self.build_frame(self.mainwin,"smb_x","smb_y","smb_width","smb_height","smb_bg",args)
		self.smb_item=[]

		#Third menu bar
		self.info=self.build_frame(self.mainwin,"info_x","info_y","info_width","info_height","info_bg",args)
		add_image(self.add_item(self.info,height=self.info.height-self.info.scrollwidth)[1],self.default_args["info_image"],width=self.info.width-self.info.scrollwidth,height=self.info.height-self.info.scrollwidth)

		#ro
		self.ro=self.build_frame(self.mainwin,"ro_x","ro_y","ro_width","ro_height","ro_bg",args)
		self.ro_item=[]
		#self.ro

		#io
		self.io=self.build_frame(self.mainwin,"io_x","io_y","io_width","io_height","io_bg",args)
		self.io_item=[]
		self.io_item_info=[]
		for i in range(self.get_pars("io_max_length",args)):
			self.io_item.append(self.add_item(self.io,height=self.get_pars("io_item_height",args),padding={"pl":0,"pt":0,"pb":0,"pr":0}))
			self.io_item_info.append(Label(self.io_item[-1][1]))
			self.io_item_info[-1].pack(anchor="nw")
			self.io_item[-1][0].pack_forget() #anchor="nw"
		self.io_oriwidth_dif=self.io_item[0][0].winfo_reqwidth()-self.io_item[0][1].winfo_reqwidth()
		self.io_open=0
		self.io_all=False

		#cpu
		self.cpu=self.build_frame(self.mainwin,"cpu_x","cpu_y","cpu_width","cpu_height","cpu_bg",args)
	def open_smb(self,event,fmb_index):
		if self.current_fmb!=None:
			for i in self.smb_item[self.current_fmb]:
				i[0].pack_forget()
		self.current_fmb=fmb_index
		reqw=0
		for i in self.smb_item[fmb_index]:
			i[0].pack(anchor="nw")
		self.smb.frame.canvas.yview_moveto(0)
	def open_ro(self,event,fmb_index,smb_index):
		if self.current_fmb2!=None and self.current_smb!=None:
			for i in self.ro_item[self.current_fmb2][self.current_smb]:
				i[0].pack_forget()
		self.current_fmb2=fmb_index
		self.current_smb=smb_index
		for i in self.ro_item[fmb_index][smb_index]:
			i[0].pack(anchor="n")
		self.ro.frame.canvas.yview_moveto(0)
	def add_first_menu(self,**args):
		self.fmb_item.append(self.add_item(self.fmb,width=self.get_pars("fmb_height",args)-self.get_pars("fmb_bd",args)))
		self.fmb_item[-1][1].config(cursor="hand2")
		self.smb_item.append([])
		self.ro_item.append([])
		fmb_index=len(self.fmb_item)-1
		self.fmb_item[fmb_index][1].bind("<Button-1>",lambda x:self.open_smb(x,fmb_index))
		return fmb_index

	def add_second_menu(self,fmb_index,**args):
		self.smb_item[fmb_index].append(self.add_item(self.smb,height=self.get_pars("smb_bh",args)))
		self.smb_item[fmb_index][-1][1].config(cursor="hand2")
		self.ro_item[fmb_index].append([])
		smb_index=len(self.smb_item[fmb_index])-1
		self.smb_item[fmb_index][smb_index][0].pack_forget()
		self.smb_item[fmb_index][smb_index][1].bind("<Button-1>",lambda x:self.open_ro(x,fmb_index,smb_index))
		return smb_index
	def add_ro(self,fmb_index,smb_index,height=None):
		self.ro_item[fmb_index][smb_index].append(self.add_item(self.ro,height=height,anchor="n"))
		ro_index=len(self.ro_item[fmb_index][smb_index])-1
		self.ro_item[fmb_index][smb_index][ro_index][0].pack_forget()
		return ro_index
	def add_item(self,frame,width="undef",height="undef",padding={},anchor="nw",side=None):
		pt=self.get_pars("ip_top",padding)
		pr=self.get_pars("ip_right",padding)
		pb=self.get_pars("ip_bottom",padding)
		pl=self.get_pars("ip_left",padding)
		srcbox=None
		box=None
		if width=="undef" and height!="undef":
			if height!=None:
				height+=2
			srcbox=frame.add_item_y(height,anchor=anchor,side=side)
			if height==None:
				box=Frame(srcbox,bd=0,width=frame.width-pr-pl-frame.scrollwidth)
			elif height-pt-pb>0:
				box=Frame(srcbox,bd=0,width=frame.width-pr-pl-frame.scrollwidth,height=height-pt-pb)
			else:
				box=Frame(srcbox,bd=0,width=frame.width-pr-pl-frame.scrollwidth,height=0)
		elif width!="undef" and height=="undef":
			if width!=None:
				width+=2
			srcbox=frame.add_item_x(width)
			if width==None:
				box=Frame(srcbox,bd=0,height=frame.height-pt-pb-frame.scrollwidth)
			elif width-pl-pr>0:
				box=Frame(srcbox,bd=0,height=frame.height-pt-pb-frame.scrollwidth,width=width-pl-pr)
			else:
				box=Frame(srcbox,bd=0,height=frame.height-pt-pb-frame.scrollwidth,width=0)
		if srcbox!=None and box!=None:
			box.place(x=pl,y=pt,anchor="nw")
			return (srcbox,box)
		return None
	def show(self):
		self.window.mainloop()
	def io_add(self):
		while self.io_state:
			try:
				if len(self.message)>0:
					message=self.message[0]
					color=self.message_color[0]
					self.message=self.message[1:]
					self.message_color=self.message_color[1:]
					self.io_item[self.io_open][0].pack_forget()
					self.io_item[self.io_open][0].pack(anchor="nw")
					text=""
					for i in message:
						text+=str(i)+" "
					if text!="":
						text=text[:-1]
					self.io_item_info[self.io_open].config(text=text,fg=color)
					reqw=self.io_item_info[self.io_open].winfo_reqwidth()
					self.io_item[self.io_open][1].config(width=reqw)
					self.io_item[self.io_open][0].config(width=reqw+self.io_oriwidth_dif)
					self.io_open+=1
					if self.io_open==len(self.io_item):
						self.io_open=0
						self.io_all=True
					self.io.frame.canvas.yview_moveto(1)
				time.sleep(0.1)
			except:
				break
	def io_recv(self,*message,color="black"):
		self.message.append(message)
		self.message_color.append(color)
if __name__=="__main__":
	CollectPages()
	#show()


from PIL import Image,ImageTk
from tkinter import Frame,Label
def get_img(file,width,height):
	img = Image.open(file).resize((width,height))
	return ImageTk.PhotoImage(img)
class add_image(Frame):
	def __init__(self,master,file,width=None,height=None,event=None,fun=None,**kw):
		Frame.__init__(self,master,**kw)
		self.master = master
		self.showImg(file,width,height,event,fun)
	def showImg(self,file,width,height,event,fun):
		if width!=None and height!=None:
			photo = get_img(file=file,width=width,height=height)
		elif width!=None and height==None:
			photo = get_img(file=file,width=width)
		elif width==None and height!=None:
			photo = get_img(file=file,height=height)
		else:
			photo = get_img(file=file)
		label = Label(self.master,image=photo)
		label.image = photo # 添加对图片的引用
		if event!=None:
			if type(event) is tuple or type(event) is list:
				for i in range(len(event)):
					label.bind(event[i],fun[i])
			else:
				label.bind(event,fun)
		label.pack()


from XenonUI.XUIlib.imgtool import add_image
from XenonUI.XUIlib.ImageLibConfig import *
from tkinter import *
from tkinter.font import Font
from tkinter import ttk,messagebox
import threading
#XUI V1.0
#Junxiang H. 2023.03.07
def update_entry(enobj,value,ro=True):
	if ro:
		enobj.config(state="normal")
	enobj.delete(0,"end")
	enobj.insert(0,value)
	if ro:
		enobj.config(state="readonly")
def retype_string(string):
	if "." in string or "e" in string:
		try:
			string=float(string)
		except:
			if string=="None":
				string=None
			elif string=="True":
				string=True
			elif string=="False":
				string=False
	else:
		try:
			string=int(string)
		except:
			pass
	return string if type(string) !=str or len(string.split(","))==1 else [retype_string(i) for i in string.split(",")]
def checkNone(string):
	if string=="None":
		return None
	return string
def str_clean(strc):
	return strc.replace(" ","_")
def str_cut(strc,length):
	new_length=length-10
	result=""
	n=0
	for i in strc:
		if i=="\t":
			n+=6
		dof=False
		if result!="" and result[-1]=="\n" and (i.upper()>"Z" or i.upper()<"A" ) and (i>"9" or i<"0"):
			result=result[:-2]
			dof=True
		if dof:
			if i!=" ":
				result+=i
			result+="\n"
		else:
			result+=i
		n+=1
		if n==new_length:
			n=0
			if (i.upper()<="Z" and i.upper()>="A" or i<="9" and i>="0"):
				result+="-"
			elif i==" ":
				result=result[:-1]
			result+="\n"
	if result[-1]=="\n":
		result=result[:-1]
	return result
def bk_fun():
	pass
def bk_fun2(strc):
	pass
def value_bind_entry(entry1,entry2,entry2_readonly=True):
	#usage:
	#fun=lambda :value_bind_entry(entry1,entry2))
	try:
		if entry2_readonly:
			entry2.config(state="normal")
		entry2.delete(0,"end")
		entry2.insert(0,entry1.get())
		if entry2_readonly:
			entry2.config(state="readonly")
	except:
		pass
def value_bind_selector(selector1,selector2):
	#usage:
	#fun=lambda :value_bind_entry(entry1,entry2))
	#try:
		if len(selector1.values)!=len(selector2.values):
			return
		for i in selector1.values:
			if i not in selector2.values:
				return
		for i in range(len(selector1.values)):
			for j in range(len(selector2.values)):
				if selector1.values[i]==selector2.values[j]:
					if selector1.var[i].get():
						selector2.cb[j].select()
					else:
						selector2.cb[j].deselect()
	#except:
	#	pass
class Selector:
	def __init__(self,frame,values,default_values=None):
		self.var=[]
		self.cb=[]
		for i in range(len(values)):
			self.var.append(BooleanVar())
			cb=Checkbutton(frame,text=values[i],variable=self.var[-1])
			cb.pack(anchor="nw")
			if default_values!=None and i in default_values:
				cb.select()
			self.cb.append(cb)
		self.values=values
	def get(self):
		result=()
		for i in range(len(self.var)):
			if self.var[i].get():
				result+=(self.values[i],)
		return result
class page:
	set_fmb=False
	def __init__(self,tkobj,**pageargs):
		self.tkobj=tkobj
		self.smb_width=self.tkobj.smb.width-self.tkobj.smb.scrollwidth
		self.smb_height=self.tkobj.smb.height-self.tkobj.smb.scrollwidth
		self.ro_width=self.tkobj.ro.width-self.tkobj.smb.scrollwidth
		self.ro_height=self.tkobj.ro.height-self.tkobj.ro.scrollwidth
		self.fmbindex=self.tkobj.add_first_menu()
		self.pageargs=pageargs
		self.initial()
		self.show()
	def add_useless_menu(self,text=""):
		smbindex=self.tkobj.add_second_menu(self.fmbindex)
		tmpframe=Frame(self.tkobj.smb_item[self.fmbindex][smbindex][1],width=self.smb_width)
		tmpframe.pack()
		lb=Label(tmpframe,text=text,bd=2,relief="solid")
		lb.pack(side="left")
		reqw=max(tmpframe.winfo_reqwidth(),lb.winfo_reqwidth())
		tmpframe.config(width=reqw)
		dd=self.tkobj.smb_item[self.fmbindex][smbindex][0].winfo_reqwidth()-self.tkobj.smb_item[self.fmbindex][smbindex][1].winfo_reqwidth()
		reqw=max(self.tkobj.smb_item[self.fmbindex][smbindex][1].winfo_reqwidth(),tmpframe.winfo_reqwidth())
		self.tkobj.smb_item[self.fmbindex][smbindex][1].unbind("<Button-1>")
		self.tkobj.smb_item[self.fmbindex][smbindex][1].config(width=reqw,cursor="arrow")
		self.tkobj.smb_item[self.fmbindex][smbindex][0].config(width=reqw+dd)
		return smbindex
	def add_menu(self,text="",submenu=0):
		text=submenu*"|----->"+text
		smbindex=self.tkobj.add_second_menu(self.fmbindex)
		tmpframe=Frame(self.tkobj.smb_item[self.fmbindex][smbindex][1],width=self.smb_width)
		tmpframe.pack()
		lb=Label(tmpframe,text=text,fg="red" if submenu==0 else "blue" if submenu==1 else "green")
		lb.pack(side="left")
		lb.bind("<Button-1>",lambda x:self.tkobj.open_ro(x,self.fmbindex,smbindex))
		reqw=max(tmpframe.winfo_reqwidth(),lb.winfo_reqwidth())
		tmpframe.config(width=reqw)
		dd=self.tkobj.smb_item[self.fmbindex][smbindex][0].winfo_reqwidth()-self.tkobj.smb_item[self.fmbindex][smbindex][1].winfo_reqwidth()
		reqw=max(self.tkobj.smb_item[self.fmbindex][smbindex][1].winfo_reqwidth(),tmpframe.winfo_reqwidth())
		self.tkobj.smb_item[self.fmbindex][smbindex][1].config(width=reqw)
		self.tkobj.smb_item[self.fmbindex][smbindex][0].config(width=reqw+dd)
		return smbindex
	def add_item(self,smbindex,height=None):
		return self.tkobj.add_ro(self.fmbindex,smbindex,height)
	def set_item(self,smbindex,roindex,tktype=Label,text=None,length=60,fontsize=18,fg="black"):
		tmp=tktype(self.tkobj.ro_item[self.fmbindex][smbindex][roindex][1],bd=0)
		try:
			tmp.config(text=str_cut(text,length),font=Font(size=fontsize),fg=fg,width=length)
		except:
			pass
		tmp.pack(side="top")
		self.update_window(tmp,smbindex,roindex)
		return tmp
	def add_row(self,smbindex,height=50,bx=50):
		item=self.add_item(smbindex,height)
		box=Frame(self.tkobj.ro_item[self.fmbindex][smbindex][item][1],width=self.tkobj.ro_item[self.fmbindex][smbindex][item][1].winfo_reqwidth()-2*bx,height=height)
		box.place(x=bx,y=0,anchor="nw")
		#return (box,self.tkobj.ro_item[self.fmbindex][smbindex][item][0])
		return box
	def add_title(self,smbindex,title,height=50,fg="blue",fontsize=16):
		#box=self.add_row(smbindex,height,bx=(self.tkobj.ro.useframe.winfo_reqwidth()-len(title)*fontsize**0.88)/2)[0]
		box=self.add_row(smbindex,height,bx=(self.tkobj.ro.useframe.winfo_reqwidth()-len(title)*fontsize**0.88)/2)
		Label(box,text=title,font=Font(size=fontsize),fg=fg,width=len(title)).pack(anchor="center")
	def update_window(self,frame,smbindex,roindex):
		dd=self.tkobj.ro_item[self.fmbindex][smbindex][roindex][0].winfo_reqwidth()-self.tkobj.ro_item[self.fmbindex][smbindex][roindex][1].winfo_reqwidth()
		self.tkobj.ro_item[self.fmbindex][smbindex][roindex][1].config(width=max(self.tkobj.ro_item[self.fmbindex][smbindex][roindex][1].winfo_reqwidth(),frame.winfo_reqwidth()))
		self.tkobj.ro_item[self.fmbindex][smbindex][roindex][0].config(width=self.tkobj.ro_item[self.fmbindex][smbindex][roindex][0].winfo_reqwidth()+dd)
	def initial(self):
		pass
	def show(self):
		pass
	def set_image(self,imgdir):
		if not self.set_fmb:
			add_image(self.tkobj.fmb_item[self.fmbindex][1],imgdir,width=self.tkobj.fmb.height-self.tkobj.fmb.scrollwidth-self.tkobj.get_pars("fmb_bd"),height=self.tkobj.fmb.height-self.tkobj.fmb.scrollwidth-self.tkobj.get_pars("fmb_bd"),event="<Button-1>",fun=lambda x:self.tkobj.open_smb(x,self.fmbindex))
			self.set_fmb=True
	def set_text(self,strc):
		if not self.set_fmb:
			lb=Label(self.tkobj.fmb_item[self.fmbindex][1],text=strc)
			lb.place(x=0,y=(self.tkobj.fmb.height-self.tkobj.fmb.scrollwidth-2*self.tkobj.get_pars("fmb_bd"))/2,anchor="nw")
			lb.bind("<Button-1>",lambda x:self.tkobj.open_smb(x,self.fmbindex))
			self.set_fmb=True
	def show_error(self,error):
		def show_errmsg():
			messagebox.showerror(title="Invalid data input",message=error)
		threading.Thread(target=show_errmsg).start()
	'''
	def page_basic(self,value_name,value_type,value_defualt,describe,check_fun=None,errmsg="",values=None,button_text="set",describe_height=150,enth=50,selh=300,bkh=50,txw=17,submenu=0,fun=bk_fun):
		dentstr="|----->"
		if submenu>0:
			submenu-=1
		menu=self.add_menu(len(dentstr)*" "*submenu+dentstr+value_name)
		self.add_item(menu,bkh)
		self.set_item(menu,self.add_item(menu,describe_height),text="\t"+describe)
		type1=(Entry,"Entry-readonly")
		type2=(ttk.Combobox,)
		type3=(Selector,)
		if value_type==None:
			return menu
		elif value_type in type1:
			inputbox=self.set_item(menu,self.add_item(menu,enth),Frame)
			Label(inputbox,text="\t\t\t\t"+value_name+": ").pack(side="left")
			value=Entry(inputbox,width=txw)
			value.insert(0,value_defualt)
			value.pack(side="left")
			if value_type=="Entry-readonly":
				value.config(state="readonly")	
		elif value_type in type2:
			inputbox=self.set_item(menu,self.add_item(menu,enth),Frame)
			Label(inputbox,text="\t\t\t\t"+value_name+": ").pack(side="left")
			value=value_type(inputbox,width=txw,values=values)
			value.pack(side="left")
			value.current(value_defualt)
		elif value_type in type3:
			inputbox=self.set_item(menu,self.add_item(menu,selh),Frame)
			Label(inputbox,text="\t\t\t\t"+value_name+": ").pack(side="left")
			f=Frame(inputbox,width=selh,height=selh,bd=0)
			f.pack(side="left")
			sel=value_type(f,values,value_defualt)
			value=sel
		if value_type not in ("Entry-readonly",):
			button=Button(inputbox,text=button_text)
			button.pack(side="left")
			def fun_button(event):
				fun() #do something
			button.bind("<Button-1>",fun_button)
		return (menu,value)
	'''


import os
XUIDir=os.path.join(__file__.rsplit("XenonUI",1)[0],"XenonUI")
XUILibDir=os.path.join(XUIDir,"XUIlib")
XUIImageLibDir=os.path.join(XUILibDir,"image")
Image_CPU=os.path.join(XUIImageLibDir,"cpu.jpeg")
Image_Disk=os.path.join(XUIImageLibDir,"disk.jpeg")
Image_RAM=os.path.join(XUIImageLibDir,"ram.png")
Image_XUI=os.path.join(XUIImageLibDir,"XUI.png")
Image_A=os.path.join(XUIImageLibDir,"A.png")
Image_B=os.path.join(XUIImageLibDir,"B.png")
Image_C=os.path.join(XUIImageLibDir,"C.png")
Image_D=os.path.join(XUIImageLibDir,"D.png")
Image_E=os.path.join(XUIImageLibDir,"E.png")
Image_F=os.path.join(XUIImageLibDir,"F.png")
Image_G=os.path.join(XUIImageLibDir,"G.png")
Image_H=os.path.join(XUIImageLibDir,"H.png")
Image_I=os.path.join(XUIImageLibDir,"I.png")
Image_J=os.path.join(XUIImageLibDir,"J.png")
Image_K=os.path.join(XUIImageLibDir,"K.png")
Image_L=os.path.join(XUIImageLibDir,"L.png")
Image_M=os.path.join(XUIImageLibDir,"M.png")
Image_N=os.path.join(XUIImageLibDir,"N.png")
Image_O=os.path.join(XUIImageLibDir,"O.png")
Image_P=os.path.join(XUIImageLibDir,"P.png")
Image_Q=os.path.join(XUIImageLibDir,"Q.png")
Image_R=os.path.join(XUIImageLibDir,"R.png")
Image_S=os.path.join(XUIImageLibDir,"S.png")
Image_T=os.path.join(XUIImageLibDir,"T.png")
Image_U=os.path.join(XUIImageLibDir,"U.png")
Image_V=os.path.join(XUIImageLibDir,"V.png")
Image_W=os.path.join(XUIImageLibDir,"W.png")
Image_X=os.path.join(XUIImageLibDir,"X.png")
Image_Y=os.path.join(XUIImageLibDir,"Y.png")
Image_Z=os.path.join(XUIImageLibDir,"Z.png")

def img_decode():
	try:
		os.rename(Image_CPU,os.path.join(XUIImageLibDir,"cpu.py"))
		os.rename(Image_Disk,os.path.join(XUIImageLibDir,"disk.py"))
		os.rename(Image_RAM,os.path.join(XUIImageLibDir,"ram.py"))
		os.rename(Image_XUI,os.path.join(XUIImageLibDir,"XUI.py"))
		os.rename(Image_A,os.path.join(XUIImageLibDir,"A.py"))
		os.rename(Image_B,os.path.join(XUIImageLibDir,"B.py"))
		os.rename(Image_C,os.path.join(XUIImageLibDir,"C.py"))
		os.rename(Image_D,os.path.join(XUIImageLibDir,"D.py"))
		os.rename(Image_E,os.path.join(XUIImageLibDir,"E.py"))
		os.rename(Image_F,os.path.join(XUIImageLibDir,"F.py"))
		os.rename(Image_G,os.path.join(XUIImageLibDir,"G.py"))
		os.rename(Image_H,os.path.join(XUIImageLibDir,"H.py"))
		os.rename(Image_I,os.path.join(XUIImageLibDir,"I.py"))
		os.rename(Image_J,os.path.join(XUIImageLibDir,"J.py"))
		os.rename(Image_K,os.path.join(XUIImageLibDir,"K.py"))
		os.rename(Image_L,os.path.join(XUIImageLibDir,"L.py"))
		os.rename(Image_M,os.path.join(XUIImageLibDir,"M.py"))
		os.rename(Image_N,os.path.join(XUIImageLibDir,"N.py"))
		os.rename(Image_O,os.path.join(XUIImageLibDir,"O.py"))
		os.rename(Image_P,os.path.join(XUIImageLibDir,"P.py"))
		os.rename(Image_Q,os.path.join(XUIImageLibDir,"Q.py"))
		os.rename(Image_R,os.path.join(XUIImageLibDir,"R.py"))
		os.rename(Image_S,os.path.join(XUIImageLibDir,"S.py"))
		os.rename(Image_T,os.path.join(XUIImageLibDir,"T.py"))
		os.rename(Image_U,os.path.join(XUIImageLibDir,"U.py"))
		os.rename(Image_V,os.path.join(XUIImageLibDir,"V.py"))
		os.rename(Image_W,os.path.join(XUIImageLibDir,"W.py"))
		os.rename(Image_X,os.path.join(XUIImageLibDir,"X.py"))
		os.rename(Image_Y,os.path.join(XUIImageLibDir,"Y.py"))
		os.rename(Image_Z,os.path.join(XUIImageLibDir,"Z.py"))
	except Exception as err:
		pass

def img_encode():
	try:
		os.rename(os.path.join(XUIImageLibDir,"cpu.py"),Image_CPU)
		os.rename(os.path.join(XUIImageLibDir,"disk.py"),Image_Disk)
		os.rename(os.path.join(XUIImageLibDir,"ram.py"),Image_RAM)
		os.rename(os.path.join(XUIImageLibDir,"XUI.py"),Image_XUI)
		os.rename(os.path.join(XUIImageLibDir,"A.py"),Image_A)
		os.rename(os.path.join(XUIImageLibDir,"B.py"),Image_B)
		os.rename(os.path.join(XUIImageLibDir,"C.py"),Image_C)
		os.rename(os.path.join(XUIImageLibDir,"D.py"),Image_D)
		os.rename(os.path.join(XUIImageLibDir,"E.py"),Image_E)
		os.rename(os.path.join(XUIImageLibDir,"F.py"),Image_F)
		os.rename(os.path.join(XUIImageLibDir,"G.py"),Image_G)
		os.rename(os.path.join(XUIImageLibDir,"H.py"),Image_H)
		os.rename(os.path.join(XUIImageLibDir,"I.py"),Image_I)
		os.rename(os.path.join(XUIImageLibDir,"J.py"),Image_J)
		os.rename(os.path.join(XUIImageLibDir,"K.py"),Image_K)
		os.rename(os.path.join(XUIImageLibDir,"L.py"),Image_L)
		os.rename(os.path.join(XUIImageLibDir,"M.py"),Image_M)
		os.rename(os.path.join(XUIImageLibDir,"N.py"),Image_N)
		os.rename(os.path.join(XUIImageLibDir,"O.py"),Image_O)
		os.rename(os.path.join(XUIImageLibDir,"P.py"),Image_P)
		os.rename(os.path.join(XUIImageLibDir,"Q.py"),Image_Q)
		os.rename(os.path.join(XUIImageLibDir,"R.py"),Image_R)
		os.rename(os.path.join(XUIImageLibDir,"S.py"),Image_S)
		os.rename(os.path.join(XUIImageLibDir,"T.py"),Image_T)
		os.rename(os.path.join(XUIImageLibDir,"U.py"),Image_U)
		os.rename(os.path.join(XUIImageLibDir,"V.py"),Image_V)
		os.rename(os.path.join(XUIImageLibDir,"W.py"),Image_W)
		os.rename(os.path.join(XUIImageLibDir,"X.py"),Image_X)
		os.rename(os.path.join(XUIImageLibDir,"Y.py"),Image_Y)
		os.rename(os.path.join(XUIImageLibDir,"Z.py"),Image_Z)
	except Exception as err:
		pass

if __name__=="__main__":
	img_decode()


#Exit.py
import os
from XenonUI.XUIlib.imgtool import add_image
from XenonUI.XUIlib.page import *
from tkinter import messagebox
class page(page):
	def initial(self):
		def fun(event):
			stt=""
			if "windows_title" in self.pageargs.keys():
				stt=" "+self.pageargs["windows_title"]
			if messagebox.askokcancel(title="Warnning",message="Exit"+stt+" ?"):
				#stop siobj.thread
				try:
					self.pageargs["XUIOBJ"].systeminfo.run=False
					self.tkobj.io_state=False
				except:
					pass
				self.tkobj.window.quit()
		add_image(self.tkobj.fmb_item[self.fmbindex][1],Image_E,width=self.tkobj.fmb.height-self.tkobj.fmb.scrollwidth-self.tkobj.get_pars("fmb_bd"),height=self.tkobj.fmb.height-self.tkobj.fmb.scrollwidth-self.tkobj.get_pars("fmb_bd"),event="<Button-1>",fun=fun)